//
// This file is part of the SDTS++ toolkit, written by the U.S.
// Geological Survey.  It is experimental software, written to support
// USGS research and cartographic data production.
// 
// SDTS++ is public domain software.  It may be freely copied,
// distributed, and modified.  The USGS welcomes user feedback, but makes
// no committment to any level of support for this code.  See the SDTS
// web site at http://mcmcweb.er.usgs.gov/sdts for more information,
// including points of contact.
//
// sc_Field.h: interface for the sc_Field class.
//
// $Id: sc_Field.h,v 1.4 1999/11/09 23:53:12 mcoletti Exp $
//

#ifndef INCLUDED_SC_FIELD_H
#define INCLUDED_SC_FIELD_H


#include <iostream>

#include <list>


#ifndef INCLUDED_SC_SUBFIELD_H
#include <sdts++/container/sc_Subfield.h>
#endif


/// Canonical container type for subfields.
typedef std::list<sc_Subfield> sc_SubfieldCntr;


class sc_Field : public sc_SubfieldCntr
{
 public:

  typedef sc_SubfieldCntr::iterator iterator;
  typedef sc_SubfieldCntr::const_iterator const_iterator;

  sc_Field();
  sc_Field( std::string const& name, std::string const& mnemonic );

  ~sc_Field();


  std::string const& getName() const;
  // Returns the SDTS Name of this field (if one has been set).

  std::string const& getMnemonic() const;
  // Returns the SDTS Mnemonic of this field (if one has been set).

  std::string const& setName( std::string const& name );
  std::string const& setMnemonic( std::string const& mnem );

 private:

  std::string name_;      // SDTS Field Name
  std::string mnemonic_;  // SDTS Field Mnemonic

  friend std::ostream& operator<<( std::ostream&, sc_Field const& );

}; // class sc_Field

#endif
